/*
 * leds.cpp
 *
 *  Created on: Apr 23, 2012
 */
#include "../FreeRTOS/FreeRTOS.h"
#include "../FreeRTOS/task.h"
#include "../drivers/tlc5940.h"
#include "../drivers/adc.h"
#include "stdio.h"

#define STEP_SIZE 0x03FF/16
#define DOUBLESTEP_SIZE 0x03FF/7

void doBasicPattern();
void doBarPattern();
void doCenterPattern();

void ledMeterTask(void *p){
	tlc5940_init(); //initialize tlc5940 pins
	adc_init(); //initialize ADC pins
	TLC5940_ClockInDC(); //write dot correction data to tlc5940

	for(;;){
		doBasicPattern(); //update gray scale data from sample
		TLC5940_SetGS_And_GS_PWM(); //write gray scale data to tlc5940
	}
}

void doBarPattern(){
	uint16_t lvl = getsample();
	int i = 0;

	for(;i<16; i++){
		tlc5940_set(i,NO_INTENSITY);
	}

	uint8_t setCh = lvl * 15/0x03FF;

	tlc5940_set(setCh,FULL_INTENSITY);
}

void doBasicPattern()
{
	int lvl = getsample();
	int i = 0;
	if (lvl > 0){
		for (; i < 16; i++){
			if ((STEP_SIZE * i) < lvl){
				tlc5940_set(i, FULL_INTENSITY);// Full Intensity
			}else{
				tlc5940_set(i, NO_INTENSITY); // Off
			}
		}
	}
}

void doCenterPattern(){
	uint16_t lvl = getsample();
	uint8_t centerLed = 7;
	int i = 0;

	for(;i < 8; i++){
		if (DOUBLESTEP_SIZE * i < lvl){
			tlc5940_set(centerLed + i, FULL_INTENSITY);// Full Intensity
			tlc5940_set(centerLed - i, FULL_INTENSITY);
		}else{
			tlc5940_set(centerLed + i, NO_INTENSITY); // Off
			tlc5940_set(centerLed - i, NO_INTENSITY);
		}
	}
}

