#include "mp3.hpp"
#include "../osHandles.h"
#include "fat/ff.h"
#include "utils/rprintf.h"
#include "drivers/sta013.h"
#include "drivers/ssp_spi.h"
#include <stdio.h>                  // printf() or iprintf()
#include <string.h>
#include "utils/vector.hpp"
#include "drivers/i2c.h"
#include "drivers/pcm1774.h"
#include "drivers/adc.h"
#define PLAYLIST_LENGTH 15


typedef enum{
	NOTHING,
	PLAY_PAUSE,
	STOP,
	NEXT,
	BACK,
	REVERSE,
	FORWARD
} mp3ctrl;

typedef enum{
	STOPPED,
	PLAYING,
	PAUSED,
	FINISHED,
	REWINDING,
	FORWARDING
}mp3playstate;


void CreatePlaylist(void *p){
	OSHANDLES *osHandles = (OSHANDLES*)p;

	FATFS SDCard;        // Takes ~550 Bytes
	if(FR_OK != f_mount(0, &SDCard)) { // Mount the Card on the File System
		rprintf("WARNING: SD CARD Could not be mounted!\n");
	}

	DIR Dir;
	FILINFO Finfo;
	FRESULT returnCode = FR_OK;

	char songs[PLAYLIST_LENGTH][13] = {{0}};

	#if _USE_LFN
	char Lfname[512];
	#endif

	char dirPath[] = "0:";
	if (FR_OK != (returnCode = f_opendir(&Dir, dirPath))) {
		rprintf("Invalid directory: |%s|\n", dirPath);
	}
	int numSongs;
	for (numSongs=0;numSongs < PLAYLIST_LENGTH;numSongs++) {
		#if _USE_LFN
		Finfo.lfname = Lfname;
		Finfo.lfsize = sizeof(Lfname);
		#endif

		char returnCode = f_readdir(&Dir, &Finfo);
		if ( (FR_OK != returnCode) || !Finfo.fname[0])
			break;
		if (!(Finfo.fattrib & AM_DIR)){

			char *p = Finfo.fname;

			strcpy(&songs[numSongs][0], p);
			rprintf("File: %s\n", &songs[numSongs][0]);
			char buf[13] = {0};
			strcpy(buf, &songs[numSongs][0]);
			char *tmpP = buf;
			strtok(tmpP,".");
			tmpP = strtok(0,"");

			if(!strcmp(tmpP, "mp3")){
				//rprintf("%s is MP3!\n", tmpP);
				//playlist.push_back();
				if (xQueueSend(osHandles->queue.mp3NameQ, &songs[numSongs], 0)){
					rprintf("Queued Successful!\n");
				}else{
					rprintf("Failed to queue song\n");
				}
			}else{
				rprintf("%s is not MP3!\n", tmpP);
			}
		}
	}

		const char portExpanderI2CAddr = 0x40;
		const char configureForOutput = 0x00;
		const char configureForInput = 0xFF;
		const char ledPortConfigNum = 6; //write to port0
		const char ledPortRegNum = 2;
		const char switchPortConfigNum = 7;//port 1
		const char switchPortRegNum = 1;
		char data = 0;
		i2cWriteDeviceRegister(portExpanderI2CAddr, ledPortConfigNum, configureForOutput);
		i2cWriteDeviceRegister(portExpanderI2CAddr, switchPortConfigNum, configureForInput);
		char lastdata = 0;
		unsigned int vol = 80;
		pcm1774_DigitalVolume(vol, vol);
	for(;;){

			data = i2cReadDeviceRegister(portExpanderI2CAddr, switchPortRegNum );
			if (data != lastdata){
				i2cWriteDeviceRegister(portExpanderI2CAddr, ledPortRegNum, data);
				lastdata = data;
				//rprintf("data: %d\n",data);
				mp3ctrl tmp = NOTHING;
				if(data & 0x80){
						tmp = PLAY_PAUSE;
						if (xQueueSend(osHandles->queue.mp3ctrl, &tmp, 0)){
							rprintf("Play CMD!\n");
						}else{
							rprintf("Failed to queue CMD\n");
						}
				}
				if(data & 0x40){
					tmp = STOP;
					if (xQueueSend(osHandles->queue.mp3ctrl, &tmp, 0)){
						rprintf("Stop CMD!\n");
					}else{
						rprintf("Failed to queue CMD\n");
					}
				}
				if(data & 0x20){
					tmp = REVERSE;
						if (xQueueSend(osHandles->queue.mp3ctrl, &tmp, 0)){
							rprintf("REVERSE CMD!\n");
						}else{
							rprintf("Failed to queue CMD\n");
						}
				}
				if(data & 0x10){
					tmp = FORWARD;
						if (xQueueSend(osHandles->queue.mp3ctrl, &tmp, 0)){
							rprintf("FORWARD CMD!\n");
						}else{
							rprintf("Failed to queue CMD\n");
						}
				}
				if(data & 0x01){

					if (vol < 100){
						vol++;
						sta013SetVolume(vol, 0);
					}
				}
				if (data & 0x02){
					if (vol > 0){
						vol--;
						sta013SetVolume(vol, 0);
					}
				}
			}
			//unsigned int sample = get_sample();
			//printf("data: %u \n", sample);
			//printf("voltage:%1.4f \n", sample*3.3 / 1023.0);
			vTaskDelay(50);
	}
}



void mp3Task(void* p)
{
	OSHANDLES *osHandles = (OSHANDLES*)p;
	mp3playstate playstate = STOPPED;
	mp3ctrl ctrlcmd=NOTHING;
	for(;;)
	{
		char songpath[17] = "0:";
		char *p = songpath;
		p = p + 2;
		char sname[13] = {0};

		if(xQueueReceive(osHandles->queue.mp3NameQ, sname, portMAX_DELAY)){
			//wait until play if not isplaying
			strcpy(p,sname);
			rprintf("Playing %s\n", sname);

			FIL fsrc;
			FRESULT res;

			res = f_open(&fsrc, songpath, FA_OPEN_EXISTING | FA_READ);
			if (res){
				rprintf("Error Opening\n");
			}else{
				playstate=PLAYING;
			}
			UINT reads = 0;
			while(1){//song open loop
				 while(playstate==STOPPED){ // stopped loop
					 if(xQueueReceive(osHandles->queue.mp3ctrl, &ctrlcmd, portMAX_DELAY)){
						 if(ctrlcmd==PLAY_PAUSE){
							 playstate = PLAYING;
							 f_close(&fsrc); //play after stop starts song over
							 res = f_open(&fsrc, songpath, FA_OPEN_EXISTING | FA_READ);
							 rprintf("Restarting\n");
						 }
					 }
				 }
				 if (playstate == REWINDING || playstate == FORWARDING){
						if (playstate == FORWARDING && reads+8 < fsrc.fsize/4096)
						{ f_lseek(&fsrc, fsrc.fptr + 4096*8); reads += 8; }
						if (playstate == REWINDING && reads-8 >= 0)
						{ f_lseek(&fsrc, fsrc.fptr - 4096*8); reads -= 8; }
						playstate = PLAYING;
				 }

				char buffer[4096] = {0};
				while(1){//read loop

					UINT bytesRead = 0;
					UINT bytesStreamed = 0;
					res = f_read(&fsrc, buffer, 4096, &bytesRead);
					reads++; //add to read for forward and reverse
					if (!res && !(bytesRead == 0)){
							while((bytesStreamed < bytesRead) && (playstate==PLAYING)) //playing loop
							{
								if(xQueueReceive(osHandles->queue.mp3ctrl, &ctrlcmd, 0)){
									if (ctrlcmd==PLAY_PAUSE){
										playstate = PAUSED;
										rprintf("Paused\n");
									}else if (ctrlcmd==STOP){
										rprintf("Stopped\n");
										playstate = STOPPED;
										break;
									}else if (ctrlcmd==REVERSE){
										playstate = REWINDING;
										rprintf("Rewind\n");
										break;
									}else if (ctrlcmd==FORWARD){
										rprintf("Forward\n");
										playstate = FORWARDING;
										break;
									}
								}else if(STA013_NEEDS_DATA())
								{
									if(xSemaphoreTake(osHandles->lock.SPI, 3000))
									{
										while((bytesStreamed < bytesRead) && STA013_NEEDS_DATA())
										{
											SELECT_MP3_CS();
											rxTxByteSSPSPI(buffer[bytesStreamed++]);
											DESELECT_MP3_CS();
										}
										xSemaphoreGive(osHandles->lock.SPI);

									}
								}
								else {
									// STA013 MP3 Decoder doesn't need data, so sleep this task
									vTaskDelay(1);
								}
							}
							if (playstate == PAUSED){
							  if(xQueueReceive(osHandles->queue.mp3ctrl, &ctrlcmd, portMAX_DELAY))//wait for play pause
								{
									if (ctrlcmd==PLAY_PAUSE){
										playstate = PLAYING;
										rprintf("Playing\n");
									}else if (ctrlcmd==STOP){
										playstate = STOPPED;
										rprintf("Stop\n");
									}else if (ctrlcmd==FORWARD){
										playstate = FORWARDING;
										rprintf("Forwarding\n");
									}else if (ctrlcmd==REVERSE){
										playstate = REWINDING;
										rprintf("Rewinding\n");
									}
								}
							}
							if(playstate == STOPPED || playstate == FORWARDING || playstate == REWINDING){
								break;
							}
					}else{
						playstate = FINISHED;
						rprintf("Finished or Error\n");
						break;
					}
					vTaskDelay(1);
				}
				if (playstate == FINISHED) break;

			}
			f_close(&fsrc);
			rprintf("Closed File\n");

		}

	}
}

