/*
 * adc.c
 *
 *  Created on: Apr 14, 2012
 *      Author: Dingo
 */

#include <stdint.h>
#include "adc.h"
#include "../System/lpc214x.h"
//#include "../FreeRTOS/task.h"
#include "../FreeRTOS/FreeRTOS.h"
#include "../FreeRTOS/task.h"
#include "../FreeRTOS/Semphr.h"
#include "utils/rprintf.h"

#define IRQ_ID_TIMER1      5
#define IRQ_SLOT_ENABLE    0x20

#define DISABLE_GLOBALPROT 0

#define SET_ADC1_PWR() PCONP |= (1<<2)
#define CONFIG_ADC1_PIN() PINSEL0 |= (3<<12)
#define CLR_BIT_CONTROL() AD1CR &= 0x00000000
#define SEL_ADC1() AD1CR |= 0x00000001
#define SET_ADC1_CLK() AD1CR |= 0x00001400
#define SET_ADC1_BURST() AD1CR |= 0x00010000
#define SET_ADC1_BIT_LEN() AD1CR &= 0xFFF1FFFF
#define SET_ADC1_PWDN() AD1CR |= 0x00200000
#define SET_ADC1_FALLINGEDGE() AD1CR &= 0xF7FFFFFF
#define START_ADC1() AD1CR |= 0x01000000

#define SET_TIMER1_RESET() T1TCR = 0x03
#define START_TIMER1() T1TCR = 0x0001


static volatile unsigned int adcSample;
//volatile unsigned int readSample;
//xSemaphoreHandle adcSampleLock;

static void adcISR() __attribute__ ((naked));


void adc_init(){
	SET_ADC1_PWR();
	adcSample = 0;
	CONFIG_ADC1_PIN();  //set pin functions p0.6 to AD1.0
	                     //pdn = 1 start = 0 edge = 0
	  CLR_BIT_CONTROL();                              // Clear All Bit Control
	  SEL_ADC1();                              // Select ADC1.0
	  SET_ADC1_CLK();                              // ADC Clock = VBP(PCLK) / 20
	  SET_ADC1_BURST();                                      // Busrt = 1 = Conversion Continue
	  SET_ADC1_BIT_LEN();                              // CLKS = 000 = 10Bit : 11 Cycle Clock Conversion
	  SET_ADC1_PWDN();                              // PDN = 1 = Active ADC Module
	  SET_ADC1_FALLINGEDGE();                              // EDGE = 0 = Conversion on Falling Edge
	  START_ADC1();                              // START = 001 = Start Conversion Now




		/******************************************************************************
		                             Timer0 (32bit)
		*******************************************************************************
		   Counter Enabled,    Counter Reset=0
		   Timer mode: count on rising edge of PCLK
		   Counter clk: 48 MHz, Counts every: 20.83 ns  (calculated with peripheral clock: 48MHz)
		   MCR0.0 : reset, generate interrupt on compare match
		*/
	   VICProtection = DISABLE_GLOBALPROT;
	   VICVectAddr5 = (long)adcISR;          /* set interrupt vector 5 */
	   VICVectCntl5 = ( IRQ_SLOT_ENABLE | IRQ_ID_TIMER1 );           /* enable slot & use it for TIMER1 Interrupt */
	   VICIntEnable |= (1<<IRQ_ID_TIMER1);           /* Enable TIMER1 Interrupt */

	   SET_TIMER1_RESET();     /* binary: 00000011 */
	   T1CTCR = 0x00;     /* binary: 00000000 */
	   T1MCR = 0x0003;     /* binary: 00000000_00000011 */
	   T1MR0 = 25000;     /* decimal 25000 500us */

	   START_TIMER1(); //Start Timer1

}

unsigned int get_sample(){
	unsigned long tmp = AD1DR;
	while(!(tmp & AD_DR_DONE)){tmp = AD1DR;}
	return ((tmp & AD_DR_RESULTMASK)>>AD_DR_RESULTSHIFT);
}

uint16_t getsample(){
	return adcSample;
}

void adcISR()
{
	portSAVE_CONTEXT();		// Save context of existing task
	T1IR = 0xFF;            // Clear interrupt source

	unsigned long tmp = AD1DR; //Read ADC1 Register
	adcSample =  ((tmp & AD_DR_RESULTMASK)>>AD_DR_RESULTSHIFT);

	VICVectAddr = 0;           // Clear the priority hardware

	portRESTORE_CONTEXT();	// Restore context
}
